/**
 * Package that contains XML-based backends which can serialize POJOs
 * to and deserialize from XML, using Stax XML parsers and generators
 * for XML processing and mostly standard Jackson data binding otherwise.
 */
package com.fasterxml.jackson.dataformat.xml;
